@@ -31,9 +31,10 @@
  */
 
 #include "squid.h"
+#include "CacheManager.h"
 #include "cbdata.h"
 #include "event.h"
-#include "CacheManager.h"
+#include "ip/tools.h"
 #include "SquidTime.h"
 #include "Store.h"
 #include "wordlist.h"
@@ -294,6 +295,7 @@ ipcacheCreateEntry(const char *name)
     static ipcache_entry *i;
     i = (ipcache_entry *)memAllocate(MEM_IPCACHE_ENTRY);
     i->hash.key = xstrdup(name);
+    Tolower(static_cast<char*>(i->hash.key));
     i->expires = squid_curtime + Config.negativeDnsTtl;
     return i;
 }
@@ -497,8 +499,7 @@ ipcacheParse(ipcache_entry *i, rfc1035_rr * answers, int nr, const char *error_m
 
     for (k = 0; k < nr; k++) {
 
-#if USE_IPV6
-        if (answers[k].type == RFC1035_TYPE_AAAA) {
+        if (Ip::EnableIpv6 && answers[k].type == RFC1035_TYPE_AAAA) {
             if (answers[k].rdlength != sizeof(struct in6_addr)) {
                 debugs(14, 1, "ipcacheParse: Invalid IPv6 address in response to '" << name << "'");
                 continue;
@@ -507,7 +508,6 @@ ipcacheParse(ipcache_entry *i, rfc1035_rr * answers, int nr, const char *error_m
             IpcacheStats.rr_aaaa++;
             continue;
         }
-#endif
 
         if (answers[k].type == RFC1035_TYPE_A) {
             if (answers[k].rdlength != sizeof(struct in_addr)) {
@@ -579,8 +579,7 @@ ipcacheParse(ipcache_entry *i, rfc1035_rr * answers, int nr, const char *error_m
             debugs(14, 3, "ipcacheParse: " << name << " #" << j << " " << i->addrs.in_addrs[j]);
             j++;
 
-#if USE_IPV6
-        } else if (answers[k].type == RFC1035_TYPE_AAAA) {
+        } else if (Ip::EnableIpv6 && answers[k].type == RFC1035_TYPE_AAAA) {
             if (answers[k].rdlength != sizeof(struct in6_addr))
                 continue;
 
@@ -590,7 +589,6 @@ ipcacheParse(ipcache_entry *i, rfc1035_rr * answers, int nr, const char *error_m
 
             debugs(14, 3, "ipcacheParse: " << name << " #" << j << " " << i->addrs.in_addrs[j] );
             j++;
-#endif
         }
 #if DNS_CNAME
         else if (answers[k].type == RFC1035_TYPE_CNAME) {
@@ -977,16 +975,12 @@ ipcacheMergeIPLists(const IpAddress *aaddrs, const int alen,
     int fc=0, t=0, c=0;
 
     IpAddress const *ip4ptrs[255];
-#if USE_IPV6
     IpAddress const *ip6ptrs[255];
-#endif
     int num_ip4 = 0;
     int num_ip6 = 0;
 
     memset(ip4ptrs, 0, sizeof(IpAddress*)*255);
-#if USE_IPV6
     memset(ip6ptrs, 0, sizeof(IpAddress*)*255);
-#endif
 
     // for each unique address in list A - grab ptr
     for (t = 0; t < alen; t++) {
@@ -999,9 +993,7 @@ ipcacheMergeIPLists(const IpAddress *aaddrs, const int alen,
                 ip4ptrs[num_ip4] = &aaddrs[t];
                 num_ip4++;
             }
-        }
-#if USE_IPV6
-        else if (aaddrs[t].IsIPv6()) {
+        } else if (aaddrs[t].IsIPv6()) {
             debugs(14,8, HERE << "A[" << t << "]=IPv6 " << aaddrs[t]);
             // check against IPv6 pruned list
             for (c = 0; c <= num_ip6; c++) {
@@ -1012,7 +1004,6 @@ ipcacheMergeIPLists(const IpAddress *aaddrs, const int alen,
                 num_ip6++;
             }
         }
-#endif
     }
 
     // for each unique address in list B - grab ptr
@@ -1026,9 +1017,7 @@ ipcacheMergeIPLists(const IpAddress *aaddrs, const int alen,
                 ip4ptrs[num_ip4] = &baddrs[t];
                 num_ip4++;
             }
-        }
-#if USE_IPV6
-        else if (baddrs[t].IsIPv6()) {
+        } else if (baddrs[t].IsIPv6()) {
             // check against IPv6 pruned list
             for (c = 0; c <= num_ip6; c++) {
                 if (ip6ptrs[c] && baddrs[t] == *ip6ptrs[c]) break; // duplicate.
@@ -1038,7 +1027,6 @@ ipcacheMergeIPLists(const IpAddress *aaddrs, const int alen,
                 num_ip6++;
             }
         }
-#endif
     }
 
     fc = num_ip6 + num_ip4;
@@ -1053,14 +1041,12 @@ ipcacheMergeIPLists(const IpAddress *aaddrs, const int alen,
 
     assert(out != NULL);
 
-#if USE_IPV6
     /* IPv6 are preferred (tried first) over IPv4 */
 
     for (int l = 0; outlen < num_ip6; l++, outlen++) {
         (*out)[outlen] = *ip6ptrs[l];
         debugs(14, 5, "ipcacheMergeIPLists:  #" << outlen << " " << (*out)[outlen] );
     }
-#endif /* USE_IPV6 */
 
     for (int l = 0; outlen < num_ip4; l++, outlen++) {
         (*out)[outlen] = *ip4ptrs[l];
@@ -1308,7 +1294,8 @@ ipcacheCycleAddr(const char *name, ipcache_addrs * ia)
         ia->cur = 0;
     }
 
-    debugs(14, 3, "ipcacheCycleAddr: " << name << " now at " << ia->in_addrs[ia->cur] << " (" << ia->cur << " of " << ia->count << ")");
+    /* NP: zero-based so we increase the human-readable number of our position */
+    debugs(14, 3, "ipcacheCycleAddr: " << name << " now at " << ia->in_addrs[ia->cur] << " (" << (ia->cur+1) << " of " << ia->count << ")");
 }
 
 /**
@@ -1351,6 +1338,28 @@ ipcacheMarkBadAddr(const char *name, IpAddress &addr)
     ipcacheCycleAddr(name, ia);
 }
 
+/// \ingroup IPCacheAPI
+void
+ipcacheMarkAllGood(const char *name)
+{
+    ipcache_entry *i;
+    ipcache_addrs *ia;
+    int k;
+
+    if ((i = ipcache_get(name)) == NULL)
+        return;
+
+    ia = &i->addrs;
+
+    /* All bad, reset to All good */
+    debugs(14, 3, "ipcacheMarkAllGood: Changing ALL " << name << " addrs to OK (" << ia->badcount << "/" << ia->count << " bad)");
+
+    for (k = 0; k < ia->count; k++)
+        ia->bad_mask[k] = 0;
+
+    ia->badcount = 0;
+}
+
 /// \ingroup IPCacheAPI
 void
 ipcacheMarkGoodAddr(const char *name, IpAddress &addr)
@@ -1439,15 +1448,11 @@ ipcacheAddEntryFromHosts(const char *name, const char *ipaddr)
     IpAddress ip;
 
     if (!(ip = ipaddr)) {
-#if USE_IPV6
         if (strchr(ipaddr, ':') && strspn(ipaddr, "0123456789abcdefABCDEF:") == strlen(ipaddr)) {
             debugs(14, 3, "ipcacheAddEntryFromHosts: Skipping IPv6 address '" << ipaddr << "'");
         } else {
             debugs(14, 1, "ipcacheAddEntryFromHosts: Bad IP address '" << ipaddr << "'");
         }
-#else
-        debugs(14, 1, "ipcacheAddEntryFromHosts: Bad IP address '" << ipaddr << "'");
-#endif
 
         return 1;
     }
@@ -1488,8 +1493,8 @@ variable_list *
 snmp_netIpFn(variable_list * Var, snint * ErrP)
 {
     variable_list *Answer = NULL;
-    debugs(49, 5, "snmp_netIpFn: Processing request:");
-    snmpDebugOid(5, Var->name, Var->name_length);
+    MemBuf tmp;
+    debugs(49, 5, "snmp_netIpFn: Processing request:" << snmpDebugOid(Var->name, Var->name_length, tmp));
     *ErrP = SNMP_ERR_NOERROR;
 
     switch (Var->name[LEN_SQ_NET + 1]) {
