@@ -1,7 +1,7 @@
 /*
  * $Id$
  *
- * DEBUG: section 0     WWW Client
+ * DEBUG: section --    WWW Client
  * AUTHOR: Harvest Derived
  *
  * SQUID Web Proxy Cache          http://www.squid-cache.org/
@@ -33,6 +33,9 @@
  */
 
 #include "config.h"
+#include "ip/IpAddress.h"
+#include "util.h"
+#include "squid_types.h"
 
 #ifdef _SQUID_MSWIN_
 /** \cond AUTODOCS-IGNORE */
@@ -84,9 +87,6 @@ using namespace Squid;
 #include <getopt.h>
 #endif
 
-#include "util.h"
-#include "ip/IpAddress.h"
-
 #ifndef BUFSIZ
 #define BUFSIZ		8192
 #endif
@@ -132,29 +132,33 @@ usage(const char *progname)
 {
     fprintf(stderr,
             "Version: %s\n"
-            "Usage: %s [-arsv] [-i IMS] [-h remote host] [-l local host] [-p port] [-m method] [-t count] [-I ping-interval] [-H 'strings'] [-T timeout] [-j 'hostheader'] [-V version] url\n"
+            "Usage: %s [-arsv] [-g count] [-h remote host] [-H 'string'] [-i IMS] [-I ping-interval] [-j 'Host-header']"
+            "[-k] [-l local-host] [-m method] [-p port] [-P file] [-t count] [-T timeout] [-u proxy-user] [-U www-user] "
+            "[-V version] [-w proxy-password] [-W www-password] url\n"
+            "\n"
             "Options:\n"
-            "    -P file      PUT request.\n"
             "    -a           Do NOT include Accept: header.\n"
+            "    -g count     Ping mode, perform \"count\" iterations (0 to loop until interrupted).\n"
+            "    -h host      Retrieve URL from cache on hostname.  Default is localhost.\n"
+            "    -H 'string'  Extra headers to send. Use '\\n' for new lines.\n"
+            "    -i IMS       If-Modified-Since time (in Epoch seconds).\n"
+            "    -I interval  Ping interval in seconds (default 1 second).\n"
+            "    -j hosthdr   Host header content\n"
+            "    -k           Keep the connection active. Default is to do only one request then close.\n"
+            "    -l host      Specify a local IP address to bind to.  Default is none.\n"
+            "    -m method    Request method, default is GET.\n"
+            "    -p port      Port number of cache.  Default is %d.\n"
+            "    -P file      PUT request. Using the named file\n"
             "    -r           Force cache to reload URL.\n"
             "    -s           Silent.  Do not print data to stdout.\n"
-            "    -v           Verbose. Print outgoing message to stderr.\n"
-            "    -i IMS       If-Modified-Since time (in Epoch seconds).\n"
-            "    -h host      Retrieve URL from cache on hostname.  Default is localhost.\n"
-            "    -l host      Specify a local IP address to bind to.  Default is none.\n"
-            "    -j hosthdr   Host header content\n"
-            "    -p port      Port number of cache.  Default is %d.\n"
-            "    -m method    Request method, default is GET.\n"
             "    -t count     Trace count cache-hops\n"
-            "    -g count     Ping mode, \"count\" iterations (0 to loop until interrupted).\n"
-            "    -I interval  Ping interval in seconds (default 1 second).\n"
-            "    -H 'string'  Extra headers to send. Use '\\n' for new lines.\n"
             "    -T timeout   Timeout value (seconds) for read/write operations.\n"
             "    -u user      Proxy authentication username\n"
-            "    -w password  Proxy authentication password\n"
             "    -U user      WWW authentication username\n"
-            "    -W password  WWW authentication password\n"
+            "    -v           Verbose. Print outgoing message to stderr.\n"
             "    -V version   HTTP Version. Use '-' for HTTP/0.9 omitted case\n",
+            "    -w password  Proxy authentication password\n"
+            "    -W password  WWW authentication password\n"
             VERSION, progname, CACHE_HTTP_PORT);
     exit(1);
 }
@@ -384,7 +388,7 @@ main(int argc, char *argv[])
             strcat(msg, buf);
         }
         if (put_fd > 0) {
-            snprintf(buf, BUFSIZ, "Content-length: %d\r\n", (int) sb.st_size);
+            snprintf(buf, BUFSIZ, "Content-length: %" PRId64 "\r\n", (int64_t) sb.st_size);
             strcat(msg, buf);
         }
         if (opt_noaccept == 0) {
@@ -430,26 +434,20 @@ main(int argc, char *argv[])
             strcat(msg, buf);
         }
 
-        /* HTTP/1.0 may need keep-alive */
-        if (strcmp(version, "1.0") == 0) {
-            if (keep_alive) {
-                if (strchr(url, ':'))
-                    snprintf(buf, BUFSIZ, "Proxy-Connection: keep-alive\r\n");
-                else
-                    strcat(msg, "Connection: keep-alive\r\n");
-            }
-        } else {
-            if (!keep_alive)
-                strcat(msg, "Connection: close\r\n");
-        }
-        strcat(msg, buf);
+        /* HTTP/1.0 may need keep-alive explicitly */
+        if (strcmp(version, "1.0") == 0 && keep_alive)
+            strcat(msg, "Connection: keep-alive\r\n");
+
+        /* HTTP/1.1 may need close explicitly */
+        if (!keep_alive)
+            strcat(msg, "Connection: close\r\n");
 
         strcat(msg, extra_hdrs);
         strcat(msg, "\r\n");
     }
 
     if (opt_verbose)
-        fprintf(stderr, "Request: '%s'\n", msg);
+        fprintf(stderr, "Request:'%s'\n", msg);
 
     if (ping) {
 #if HAVE_SIGACTION
@@ -477,6 +475,9 @@ main(int argc, char *argv[])
         int fsize = 0;
         struct addrinfo *AI = NULL;
 
+        if (opt_verbose)
+            fprintf(stderr, "Resolving... %s\n", hostname);
+
         /* Connect to the server */
 
         if (localhost) {
@@ -514,18 +515,27 @@ main(int argc, char *argv[])
 
         iaddr.SetPort(port);
 
+        if (opt_verbose) {
+            char ipbuf[MAX_IPSTRLEN];
+            fprintf(stderr, "Connecting... %s(%s)\n", hostname, iaddr.NtoA(ipbuf, MAX_IPSTRLEN));
+        }
+
         if (client_comm_connect(conn, iaddr, ping ? &tv1 : NULL) < 0) {
-            char buf[MAX_IPSTRLEN];
-            iaddr.ToURL(buf, MAX_IPSTRLEN);
+            char hostnameBuf[MAX_IPSTRLEN];
+            iaddr.ToURL(hostnameBuf, MAX_IPSTRLEN);
             if (errno == 0) {
-                fprintf(stderr, "client: ERROR: Cannot connect to %s: Host unknown.\n", buf);
+                fprintf(stderr, "client: ERROR: Cannot connect to %s: Host unknown.\n", hostnameBuf);
             } else {
                 char tbuf[BUFSIZ];
-                snprintf(tbuf, BUFSIZ, "client: ERROR: Cannot connect to %s", buf);
+                snprintf(tbuf, BUFSIZ, "client: ERROR: Cannot connect to %s", hostnameBuf);
                 perror(tbuf);
             }
             exit(1);
         }
+        if (opt_verbose) {
+            char ipbuf[MAX_IPSTRLEN];
+            fprintf(stderr, "Connected to: %s (%s)\n", hostname, iaddr.NtoA(ipbuf, MAX_IPSTRLEN));
+        }
 
         /* Send the HTTP request */
         bytesWritten = mywrite(conn, msg, strlen(msg));
@@ -633,7 +643,8 @@ main(int argc, char *argv[])
 }
 
 static int
-client_comm_bind(int sock, const IpAddress &addr) {
+client_comm_bind(int sock, const IpAddress &addr)
+{
 
     int res;
 
@@ -651,7 +662,8 @@ client_comm_bind(int sock, const IpAddress &addr) {
 }
 
 static int
-client_comm_connect(int sock, const IpAddress &addr, struct timeval *tvp) {
+client_comm_connect(int sock, const IpAddress &addr, struct timeval *tvp)
+{
     int res;
     static struct addrinfo *AI = NULL;
 
@@ -670,7 +682,8 @@ client_comm_connect(int sock, const IpAddress &addr, struct timeval *tvp) {
 }
 
 static int
-Now(struct timeval *tp) {
+Now(struct timeval *tp)
+{
 #if GETTIMEOFDAY_NO_TZP
     return gettimeofday(tp);
 #else
@@ -680,18 +693,21 @@ Now(struct timeval *tp) {
 }				/* ARGSUSED */
 
 static void
-catchSignal(int sig) {
+catchSignal(int sig)
+{
     interrupted = 1;
     fprintf(stderr, "Interrupted.\n");
 }
 
 static void
-pipe_handler(int sig) {
+pipe_handler(int sig)
+{
     fprintf(stderr, "SIGPIPE received.\n");
 }
 
 static void
-set_our_signal(void) {
+set_our_signal(void)
+{
 #if HAVE_SIGACTION
 
     struct sigaction sa;
@@ -711,7 +727,8 @@ set_our_signal(void) {
 }
 
 static ssize_t
-myread(int fd, void *buf, size_t len) {
+myread(int fd, void *buf, size_t len)
+{
 #ifndef _SQUID_MSWIN_
     alarm(io_timeout);
     return read(fd, buf, len);
@@ -722,7 +739,8 @@ myread(int fd, void *buf, size_t len) {
 }
 
 static ssize_t
-mywrite(int fd, void *buf, size_t len) {
+mywrite(int fd, void *buf, size_t len)
+{
 #ifndef _SQUID_MSWIN_
     alarm(io_timeout);
     return write(fd, buf, len);
